@@ -20,7 +20,7 @@
 #ifndef GDNSD_NET_PRIV_H
 #define GDNSD_NET_PRIV_H
 
-#include "gdnsd/net.h"
+#include <gdnsd/net.h>
 
 // This is for the core code only, called early and single-threaded.
 void gdnsd_init_net(void);
