@@ -27,36 +27,31 @@
 #include <string.h>
 #include <unistd.h>
 
-#include "gdnsd/net.h"
-#include "gdnsd/net-priv.h"
-#include "gdnsd/log.h"
+#include <gdnsd/net.h>
+#include <gdnsd/net-priv.h>
+#include <gdnsd/log.h>
 
 /* network utils */
 
 static int tcp_proto = 0;
 static int udp_proto = 0;
-static bool tcp_v6_ok = false;
 static bool reuseport_ok = false;
 
 void gdnsd_init_net(void) {
     struct protoent* pe;
 
+    // cppcheck-suppress nonreentrantFunctionsgetprotobyname (init time, no threads)
     pe = getprotobyname("tcp");
     if(!pe)
         log_fatal("getprotobyname('tcp') failed");
     tcp_proto = pe->p_proto;
 
+    // cppcheck-suppress nonreentrantFunctionsgetprotobyname (init time, no threads)
     pe = getprotobyname("udp");
     if(!pe)
         log_fatal("getprotobyname('udp') failed");
     udp_proto = pe->p_proto;
 
-    const int sock_v6 = socket(PF_INET6, SOCK_STREAM, tcp_proto);
-    if(sock_v6 > -1) {
-        close(sock_v6);
-        tcp_v6_ok = true;
-    }
-
 #ifdef SO_REUSEPORT
     const int sock_rp = socket(PF_INET, SOCK_DGRAM, udp_proto);
     if(sock_rp > -1) {
@@ -70,5 +65,4 @@ void gdnsd_init_net(void) {
 
 int gdnsd_getproto_udp(void) { return udp_proto; }
 int gdnsd_getproto_tcp(void) { return tcp_proto; }
-bool gdnsd_tcp_v6_ok(void) { return tcp_v6_ok; }
 bool gdnsd_reuseport_ok(void) { return reuseport_ok; }
