@@ -21,14 +21,15 @@
 #define GDNSD_ZSRC_RFC1035_H
 
 #include "config.h"
-#include "gdnsd/compiler.h"
+#include <gdnsd/compiler.h>
 #include <ev.h>
+#include <stdbool.h>
 
-void zsrc_rfc1035_load_zones(void);
+void zsrc_rfc1035_load_zones(const bool check_only);
 
 F_NONNULL
 void zsrc_rfc1035_runtime_init(struct ev_loop* loop);
 
-void zsrc_rfc1035_sighup(void);
+void zsrc_rfc1035_sigusr1(void);
 
 #endif // GDNSD_ZSRC_RFC1035_H
