@@ -1,6 +1,6 @@
 /* Copyright © 2012 Brandon L Black <blblack@gmail.com>
  *
- * This file is part of gdnsd-plugin-geoip.
+ * This file is part of gdnsd.
  *
  * gdnsd-plugin-geoip is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 
 #include "config.h"
 #include "ntree.h"
+#include <gdnsd/alloc.h>
 #include <gdnsd/log.h>
 
 // Initial node allocation count,
@@ -26,8 +27,8 @@
 static const unsigned NT_SIZE_INIT = 128;
 
 ntree_t* ntree_new(void) {
-    ntree_t* newtree = malloc(sizeof(ntree_t));
-    newtree->store = malloc(NT_SIZE_INIT * sizeof(nnode_t));
+    ntree_t* newtree = xmalloc(sizeof(ntree_t));
+    newtree->store = xmalloc(NT_SIZE_INIT * sizeof(nnode_t));
     newtree->count = 0;
     newtree->alloc = NT_SIZE_INIT; // set to zero on fixation
     return newtree;
@@ -44,7 +45,7 @@ unsigned ntree_add_node(ntree_t* tree) {
     dmn_assert(tree->alloc);
     if(tree->count == tree->alloc) {
         tree->alloc <<= 1;
-        tree->store = realloc(tree->store, tree->alloc * sizeof(nnode_t));
+        tree->store = xrealloc(tree->store, tree->alloc * sizeof(nnode_t));
     }
     const unsigned rv = tree->count;
     dmn_assert(rv < (1U << 24));
@@ -74,7 +75,7 @@ static unsigned ntree_find_v4root(const ntree_t* tree) {
 void ntree_finish(ntree_t* tree) {
     dmn_assert(tree);
     tree->alloc = 0; // flag fixed, will fail asserts on add_node, etc now
-    tree->store = realloc(tree->store, tree->count * sizeof(nnode_t));
+    tree->store = xrealloc(tree->store, tree->count * sizeof(nnode_t));
     tree->ipv4 = ntree_find_v4root(tree);
 }
 
@@ -87,12 +88,12 @@ F_NONNULL
 static void ntree_dump_rec_sub(const ntree_t* tree, const unsigned bitdepth, const unsigned val, struct in6_addr ipv6) {
     dmn_assert(tree);
     if(NN_IS_DCLIST(val)) {
-        anysin_t tempsin;
+        dmn_anysin_t tempsin;
         memset(&tempsin, 0, sizeof(tempsin));
         tempsin.len = sizeof(struct sockaddr_in6);
         tempsin.sa.sa_family = AF_INET6;
         memcpy(&tempsin.sin6.sin6_addr, &ipv6, sizeof(struct in6_addr));
-        log_debug("%s/%u -> %u", logf_anysin_noport(&tempsin), 128U - bitdepth, NN_GET_DCLIST(val));
+        log_debug("%s/%u -> %u", dmn_logf_anysin_noport(&tempsin), 128U - bitdepth, NN_GET_DCLIST(val));
     }
     else {
         dmn_assert(bitdepth);
@@ -137,7 +138,7 @@ void ntree_assert_optimal(const ntree_t* tree) {
 #endif
 
 F_NONNULL
-static inline bool CHKBIT_v6(const uint8_t* ipv6, const unsigned bit) {
+static bool CHKBIT_v6(const uint8_t* ipv6, const unsigned bit) {
     dmn_assert(ipv6);
     dmn_assert(bit < 128);
     return ipv6[bit >> 3] & (1UL << (~bit & 7));
@@ -162,7 +163,7 @@ static unsigned ntree_lookup_v6(const ntree_t* tree, const uint8_t* ip, unsigned
     return NN_GET_DCLIST(offset);
 }
 
-static inline bool CHKBIT_v4(const uint32_t ip, const unsigned maskbit) {
+static bool CHKBIT_v4(const uint32_t ip, const unsigned maskbit) {
     dmn_assert(maskbit < 32U);
     return ip & (1U << (31U - maskbit));
 }
@@ -220,7 +221,7 @@ static uint32_t v6_v4fixup(const uint8_t* in, unsigned* mask_adj) {
 }
 
 F_NONNULL
-static unsigned ntree_lookup_inner(const ntree_t* tree, const anysin_t* client_addr, unsigned* scope_mask) {
+static unsigned ntree_lookup_inner(const ntree_t* tree, const dmn_anysin_t* client_addr, unsigned* scope_mask) {
     dmn_assert(tree); dmn_assert(client_addr); dmn_assert(scope_mask);
 
     unsigned rv;
